package syhd.cas.bean;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.jasig.cas.client.Protocol;
import org.jasig.cas.client.authentication.AuthenticationFilter;
import org.jasig.cas.client.util.AbstractCasFilter;
import org.jasig.cas.client.util.CommonUtils;
import org.jasig.cas.client.validation.Assertion;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseCookie;
import org.springframework.web.filter.OncePerRequestFilter;
import sun.net.util.URLUtil;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

/**
 * @author zhangbaisen
 * @date 2022/2/25 15:25
 */
public class SyhdCasAuthFilter extends OncePerRequestFilter {
    private final ObjectMapper mapper = new ObjectMapper();
    public static final String ticket = "ticket";
    public static final String targetUrl = "targetUrl";
    
    private String serverLoginUrl;
    private String serviceUrlPrefix;
    private String casRedirect;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        String url = request.getHeader(targetUrl);
        if(!isAbsoluteUrl(url)){
            filterChain.doFilter(request, response);
            return;
        }
        
        HttpSession session = request.getSession(false);
        final Assertion assertion = session != null ? (Assertion) session.getAttribute(AbstractCasFilter.CONST_CAS_ASSERTION) : null;

        if (assertion != null) {
            filterChain.doFilter(request, response);
            return;
        }

        final String ticket = retrieveTicketFromRequest(request);
        if (CommonUtils.isNotBlank(ticket)) {
            filterChain.doFilter(request, response);
            return;
        }

        
        if(session == null){
            session = request.getSession();
            
            System.out.println("SyhdCasAuthFilter   sessionId " + session.getId());
        }
        session.setAttribute(targetUrl,url);
        
        String redirectUrl = this.serverLoginUrl + "?service=" + this.serviceUrlPrefix + this.casRedirect;
        response.setCharacterEncoding("utf-8");
        response.setContentType("application/json; charset=utf-8");
        response.setStatus(HttpStatus.UNAUTHORIZED.value());

        PrintWriter writer = response.getWriter();
        Map<String, Object> map = new HashMap<>();
        map.put("url",redirectUrl);
//        map.put("code",HttpStatus.UNAUTHORIZED.value());
        String json = mapper.writeValueAsString(map);
        writer.write(json);
    }

    protected String retrieveTicketFromRequest(final HttpServletRequest request) {
        if ("POST".equals(request.getMethod())) {
            return request.getParameter(ticket);
        }
        return request.getQueryString() == null || !request.getQueryString().contains(ticket) ? null : request
                .getParameter(ticket);
    }

    public static boolean isAbsoluteUrl(String url) {
        if (url == null) {
            return false;
        }
        final Pattern ABSOLUTE_URL = Pattern.compile("\\A[a-z0-9.+-]+://.*",
                Pattern.CASE_INSENSITIVE);

        return ABSOLUTE_URL.matcher(url).matches();
    }

    public void setServerLoginUrl(String serverLoginUrl) {
        this.serverLoginUrl = serverLoginUrl;
    }

    public void setServiceUrlPrefix(String serviceUrlPrefix) {
        this.serviceUrlPrefix = serviceUrlPrefix;
    }

    public void setCasRedirect(String casRedirect) {
        this.casRedirect = casRedirect;
    }
}
